@@ -1,4 +1,5 @@
 #!/usr/bin/python
+#Import necessary modules
 import os
 import sys,time,re
 import webbrowser
@@ -13,7 +14,7 @@ import dbus
 from dbus.mainloop.glib import DBusGMainLoop
 
 __title__ = "Pandora"
-
+#class for the "sleep" function -- needs a way to cancel properly
 class snooze(QtCore.QThread):
 	__pyqtSignals__ = ( "shutdown" )
 	def __init__(self,ktime,cont=None):
@@ -25,6 +26,7 @@ class snooze(QtCore.QThread):
 		if self.cont:
 			self.emit(QtCore.SIGNAL("shutdown"))
 
+#class for reading configuration files
 class Config:
 	def __init__(self):
 		self.user = ""
@@ -32,9 +34,10 @@ class Config:
 		self.color = ""
 		self.lastplayed = ""
 		self.format = "mp3"
+		#I loop through possible configuration options in this array
 		self.config_opts = ['user','password','format']
 
-	def write(self):
+	def write(self): #write the configuration files
 		if not os.path.exists(os.path.expanduser("~")+'/.config'):
 			os.mkdir(os.path.expanduser("~")+'/.config')
 
@@ -45,8 +48,8 @@ class Config:
 		f.write(newfile)
 		f.close()
 
-	def load(self):
-		if os.path.exists('/etc/hildon/theme/colors.config'):
+	def load(self): #load the configuration files
+		if os.path.exists('/etc/hildon/theme/colors.config'): #get system colors
 			f = open('/etc/hildon/theme/colors.config')
 			for line in f:
 				if re.match("ActiveTextColor\s*=",line):
@@ -55,22 +58,24 @@ class Config:
 			if self.color == "":
 				self.color = "rgb(255, 255, 255)"
 
-		else:
+		else: #default to white
 			self.color = "rgb(255, 255, 255)"
 		
+		#open main configuration file
 		if os.path.exists(os.path.expanduser("~")+'/.config/pyradio/config'):
 			f = open(os.path.expanduser("~")+'/.config/pyradio/config', 'r')
 			for line in f:
 				line = line.replace(" ", "")
 				
-				if re.match("^#.*",line) is not None:
+				if re.match("^#.*",line) is not None: #skip comments
 					continue
 				
-				for opt in self.config_opts:
+				for opt in self.config_opts: #check each line for an option
 					if re.match("^"+opt+"=(.+)$",line) is not None:
 						setattr(self,opt,self.get_var(opt,line))
 			f.close()
 
+		#get lastplayed station to auto-start
 		if os.path.exists(os.path.expanduser("~")+'/.config/pyradio/lastplayed'):
 			f = open(os.path.expanduser("~")+'/.config/pyradio/lastplayed', 'r')
 			self.lastplayed = f.readline().replace("\n","")
@@ -80,31 +85,26 @@ class Config:
 		r = re.match("^"+var+"=(.+)$",line)
 		return r.group(1)
 		
-class Panda(QtGui.QMainWindow):
+class Panda(QtGui.QMainWindow): #main class
 	def __init__( self, parent=None ):
 		QtGui.QWidget.__init__(self, parent)
-		self.ui = None
-		self.pandora = None
-		self.die = False
+		self.ui = None 
 		self.authenticated = False
-		self.c = Config()
+		self.c = Config() 
 		self.c.load()
 		self.player = RadioPlayer()
 		self.player.format=self.c.format
 		self.player.start()
-		self.pui = Ui_pyRadio_portrait()
-		self.pui.setupUi(self)
-		self.pui = None
 		self.ui = Ui_pyRadio()
 		self.ui.setupUi(self)
 		self.portrait = False
 		self.os = self.player.get_product()
-		if self.os != "RX-51":
+		if self.os != "RX-51": #check for N900 
 			self.ui.albumArt.setStyleSheet("background-color: rgb(255, 255, 255);")
 
-		if not self.c.lastplayed == "":
+		if not self.c.lastplayed == "": #auto-play last station
 			self.player.playStation = self.c.lastplayed
-		try :
+		try : #test for rotation and set auto-rotation on
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 			self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged );
 			screenGeometry=QApplication.desktop().screenGeometry();
@@ -118,9 +118,9 @@ class Panda(QtGui.QMainWindow):
 		self.s = snooze(0)
 		self.createStations = []
 		self.count = 0
-		self.connect_signals()
+		self.connect_signals() #called every time orientation changes to connect slots
 		self.auth()
-		try :
+		try : #test for FM transmitter.
 			sysbus = dbus.SystemBus()
 			self.fmtx = sysbus.get_object('com.nokia.FMTx', '/com/nokia/fmtx/default', False)
 			self.fmtx_iface = dbus.Interface(self.fmtx, dbus_interface='org.freedesktop.DBus.Properties')
@@ -130,11 +130,11 @@ class Panda(QtGui.QMainWindow):
 				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 			else:
 				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
-		except: print "Not N900."
+		except: print "No FM Transmitter."
 
-	def orientationChanged ( self ):
+	def orientationChanged ( self ): #called when N900 is rotated
 		screenGeometry=QApplication.desktop().screenGeometry();
-		if screenGeometry.width() > screenGeometry.height():
+		if screenGeometry.width() > screenGeometry.height(): #landscape
 			self.ui = Ui_pyRadio()
 			self.ui.setupUi(self)
 			self.connect_signals()
@@ -142,7 +142,7 @@ class Panda(QtGui.QMainWindow):
 			if not self.player.curSong == 0:
 				self.setUi ( self.player.curSong )
 				self.buildPlaylist()
-		else:
+		else: #portrait
 			self.ui = Ui_pyRadio_portrait()
 			self.ui.setupUi(self)
 			self.connect_signals()
@@ -152,7 +152,7 @@ class Panda(QtGui.QMainWindow):
 				self.buildPlaylist()
 		print "Orientation Changed"
 
-	def connect_signals( self ):
+	def connect_signals( self ): #connect all signals
 		if not self.authenticated:
 			self.ui.btnPlay.setEnabled( False )
 			self.ui.listStations.setEnabled( False )
@@ -188,8 +188,42 @@ class Panda(QtGui.QMainWindow):
 		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.createStation)
 		QtCore.QObject.connect(self.ui.actionDonate, QtCore.SIGNAL('triggered()'), self.open_donate)
 		QtCore.QObject.connect(self.ui.actionQuality, QtCore.SIGNAL('triggered()'), self.open_quality)
+		QtCore.QObject.connect(self.ui.actionAdd_Seed, QtCore.SIGNAL('triggered()'), self.doSeed)
 
-	def open_donate( self ):
+	def doSeed( self ):
+		if not self.player.playing:
+			QtGui.QMessageBox.warning(self, "Not Playing", "You must be listening to a song to seed it.")
+			return
+
+		self.d = QDialog(self)
+		self.d.setParent(self)
+		if self.os == "RX-51":
+			if not self.portrait:
+				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
+			else:
+				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
+		vbox = QVBoxLayout()
+		hLayout = QHBoxLayout()
+		m = QPushButton("Seed Artist")
+		p = QPushButton("Seed Song")
+		c = QPushButton("Close")
+		hLayout.addWidget(m)
+		hLayout.addWidget(p)
+		hLayout.addWidget(c)
+		m.clicked.connect(lambda: self.addSeed("artist"))
+		p.clicked.connect(lambda: self.addSeed("song"))
+		c.clicked.connect(self.d.close)
+		vbox.addLayout(hLayout)
+		self.d.setLayout(vbox)
+		self.d.setWindowTitle("Add Station Seed:")
+		self.d.show()
+
+	def addSeed( self, seed=None ):
+		if self.player.playing:
+			self.player.seedSong = seed
+		self.d.close()
+ 
+	def open_donate( self ): #Donations are AWESOME :)
 		self.d = QDialog(self)
 		self.d.setParent(self)
 		if self.os == "RX-51":
@@ -200,7 +234,7 @@ class Panda(QtGui.QMainWindow):
 		vbox = QVBoxLayout()
 		hLayout = QHBoxLayout()
 		m = QPushButton("pyRadio")
-		p = QPushButton("Get Pandora One")
+		p = QPushButton("Get Pandora One") #Let's support our radio host :)
 		c = QPushButton("Close")
 		hLayout.addWidget(m)
 		hLayout.addWidget(p)
@@ -215,7 +249,7 @@ class Panda(QtGui.QMainWindow):
 		self.d.setWindowTitle("Donate to:")
 		self.d.show()
 
-	def open_quality( self ):
+	def open_quality( self ): #Quality Selector
 		self.d = QDialog(self)
 		self.d.setParent(self)
 		if self.os == "RX-51":
@@ -239,12 +273,12 @@ class Panda(QtGui.QMainWindow):
 		self.d.setWindowTitle("Sound Quality:")
 		self.d.show()
 
-	def setFormat(self, format="mp3"):
+	def setFormat(self, format="mp3"): #write quality out to the config file so we can reload later
 		self.c.format=format
 		self.c.write()
 		self.player.format=format
 		self.player.playlist = []
-		self.player.getPlaylist = 1
+		self.player.getPlaylist = 1 #Get a new playlist with the new format!
 		self.d.close()
 
 	def donate( self, who=None ):
@@ -254,17 +288,17 @@ class Panda(QtGui.QMainWindow):
 			webbrowser.open_new('http://www.pandora.com/pandora_one')
 		self.d.close()
 
-	def stationCreated( self ):
+	def stationCreated( self ): #Station Created Successfully
 		ok = QtGui.QMessageBox.warning(self,'Station Created','Station Created, your station list will reload.')
 		self.ui.listStations.clear()
 		self.player.getStatlist = 1
 	
-	def setDuration( self ):
+	def setDuration( self ): #Set the song position/duration
 		if (self.player.duration-self.player.position) < self.player.duration:
 			position = "%02d:%02d" % divmod(self.player.duration - self.player.position, 60) + " / " + "%02d:%02d" % divmod(self.player.duration, 60)
 			self.ui.lblTime.setText(position)
 
-	def createStation( self ):
+	def createStation( self ): #Start the create station sequence
 		if self.os == "RX-51":
 			if not self.portrait:
 				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
@@ -291,7 +325,8 @@ class Panda(QtGui.QMainWindow):
 						songs = ret["songs"]
 						title = "songTitle"
 
-				self.ui.listStations.clear()
+				self.ui.listStations.clear() 
+				#For lack of room, populate station list with new stations.  I want to change this to be it's own QDialog in the future.
 				for s in songs:
 					self.ui.listStations.addItem(s[title])
 					self.createStations = songs
@@ -299,14 +334,14 @@ class Panda(QtGui.QMainWindow):
 		if self.os == "RX-51":
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 
-	def stationDeleted( self ):
+	def stationDeleted( self ): #Delete station
 		ok = QtGui.QMessageBox.warning(self, "Station Deleted", "Station Deleted.")
 
-	def sigTired( self ):
+	def sigTired( self ): #Song marked tired successfully 
 		ok = QtGui.QMessageBox.warning(self, "Tired", "Song marked tired.")
 		self.skipSong()
 
-	def deleteStation(self):
+	def deleteStation(self): #Start the delete station process
 		if self.os == "RX-51":
 			if not self.portrait:
 				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
@@ -322,12 +357,12 @@ class Panda(QtGui.QMainWindow):
 		if self.os == "RX-51":
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 
-	def tiredSong(self):
+	def tiredSong(self): #Start tired song process and wait for signal from radio backend
 		check = QtGui.QMessageBox.question(self, 'Are you sure?', 'Are you tired of the song ' + self.player.curSong[1]["title"] + '?','No','Yes')
 		if check == 1:
 			self.player.tiredSong = 1
 
-	def setSnooze(self):
+	def setSnooze(self): #Set tired
 		if self.os == "RX-51":
 			if not self.portrait:
 				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
@@ -341,7 +376,7 @@ class Panda(QtGui.QMainWindow):
 		if self.os == "RX-51":
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 
-	def doFM(self):
+	def doFM(self): #Turn on FM transmitter
 		state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
 		if state == 'disabled':
 			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
@@ -350,13 +385,13 @@ class Panda(QtGui.QMainWindow):
 			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))
 
-	def nextDown(self):
+	def nextDown(self): #change button color
 		self.ui.btnNext.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/skip-trans.png);")
 
-	def nextUp(self):
+	def nextUp(self): #change button color
 		self.ui.btnNext.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/skip-trans.png);")
 
-	def doPlayPause( self, signal=None ):
+	def doPlayPause( self, signal=None ): #Play if stopped, pause if playing, receive signals from Backend if changed from somewhere else (ie: headphones)
 		if self.player.playing:
 			if signal != "play":
 				self.player.stopPlayBack('pause')
@@ -372,34 +407,35 @@ class Panda(QtGui.QMainWindow):
 			self.player.playing = 1
 			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/playing-trans.png);")
 
-	def doLove( self ):
+	def doLove( self ): #Love this song
 		self.player.loveHateRating = True
 		self.player.loveHateSong = 1
 		self.player.curSong[1]["rating"] = 1
 		self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
 		"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Thumbs-up-right-trans.png);")
 
-	def doHate( self ):
+	def doHate( self ): #Hate this song
 		self.player.loveHateRating = False
 		self.player.loveHateSong = 1
 		self.skip = 1
 
-	def sigLoveHate( self ):
+	def sigLoveHate( self ): #Signal for Love/Hate from backend
 		if not self.player.respLH:
 			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
 			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Thumbs-up-right.png);")
 		if self.skip:
 			self.skipSong()
 
-	def hateDown(self):
+	def hateDown(self): #change button color
 		self.ui.btnHate.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbs-down-trans.png);")
 
-	def hateUp(self):
+	def hateUp(self): #change button color
 		self.ui.btnHate.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbs-down-trans.png);")
 
-	def sigAuth( self ):
+	def sigAuth( self ): #Signal for authentication success/failure.
 		if self.player.success:
 			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication succeeded.  Wait for station list to appear in drop down below and select a station.</font></td><tr></table>" )
+			self.count = 0
 			self.player.getStatlist = 1
 		else:
 			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication failed.  Please try again.</font></td><tr></table>" )
@@ -407,7 +443,7 @@ class Panda(QtGui.QMainWindow):
 			self.c.password = ""
 			self.auth()
 
-	def auth( self ):
+	def auth( self ): #Start authentication process.
 		if self.os == "RX-51":
 			if not self.portrait:
 				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
@@ -416,12 +452,15 @@ class Panda(QtGui.QMainWindow):
 		if self.count>2:
 			self.closeEvent()
 		self.count += 1
+		ok = True #In case we got the user/pass from config
 		if self.c.user == "":
 			self.c.user,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:", QLineEdit.Normal)
-		if self.c.password == "":
+		if not ok: #Username was cancelled.. they must be done.
+			self.closeEvent()
+		if self.c.password == "": 
 			self.c.password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:", QLineEdit.Password)
-		if self.c.user == "" or self.c.password == "":
-			self.auth()
+		if not ok: #Password was cancelled.. they must be done.
+			self.closeEvent()
 		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authenticating to Pandora as "+self.c.user+" ... </font></td><tr></table>" )
 		self.player.user = self.c.user
 		self.player.password = self.c.password
@@ -431,8 +470,8 @@ class Panda(QtGui.QMainWindow):
 		if self.os == "RX-51":
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 
-	def playStation( self ):
-		if len(self.createStations)>0:
+	def playStation( self ): #Play currently selected station
+		if len(self.createStations)>0: #If we're creating a station, finish creating and not playing selected station
 			self.player.createStationId = self.createStations[self.ui.listStations.currentIndex()]['musicId']
 			self.createStations = []
 			return
@@ -455,7 +494,7 @@ class Panda(QtGui.QMainWindow):
 			f.write(self.player.curStation)
 			f.close()
 
-	def getStations( self ):	
+	def getStations( self ): #Get Station List	
 		self.count = 0
 		self.ui.listStations.clear()
 		self.ui.btnPlay.setEnabled( True )
@@ -470,19 +509,19 @@ class Panda(QtGui.QMainWindow):
 		if self.player.playStation > 0:
 			self.playStation()
 
-	def buildPlaylist( self ):
-		for s in self.player.stations:
+	def buildPlaylist( self ): #Populate the station list box
+		for s in self.player.stations: #Stations need to be scanned for QuickMix seperately
 			if s["isQuickMix"]:
 				qMixIds = s["quickMixStationIds"]
 				break
 
-		for s in self.player.stations:
+		for s in self.player.stations: #Add stations
 			#print s
 			#print ""
 			sName = s["stationName"]
 			for qMixId in qMixIds:
 				if qMixId == s["stationId"]:
-					sName += " *"
+					sName += " *" #If we're in the QuickMix list we add a *
 			self.ui.listStations.addItem(sName)
 			if self.player.playStation>0 and s["stationId"] == self.player.playStation:
 				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
@@ -490,18 +529,18 @@ class Panda(QtGui.QMainWindow):
 			if self.player.curStation>0 and s["stationId"] == self.player.curStation:
 				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
 
-	def setUserPass( self ):
+	def setUserPass( self ): #Reset username/password and start authenticating
 		self.c.user = ""
 		self.c.password = ""
 		self.auth()
 
-	def playNextSong( self ):
+	def playNextSong( self ): #Go to next song in playlist
 		if not self.player.playing:
 			raise PandaException()
 		while self.player.playlist == []:
-			time.sleep( 0.5 )
+			self.player.getMoreSongs()
 		next = self.player.playlist.pop( 0 )
-		print next
+		#print next
 		self.player.curSong = next
 		self.player.startPlay = 1
 		self.setUi ( next )
@@ -509,7 +548,7 @@ class Panda(QtGui.QMainWindow):
 			#Out of songs, grab some more while playing
 			self.player.getPlaylist = 1
 	
-	def setUi ( self, next=None ):
+	def setUi ( self, next=None ): #Update the album graphic and set Love graphic
 		art = next[1]["art"]
 		self.ui.albumArt.setHtml("<table width='100%' height='100%'><tr><td vertical-align='middle' style='text-align: center'><img style='width: 80%; height: 80%' src='"+art+"'></td></tr></table>")
 		self.ui.lblAlbum.setText("\"" + next[1]["title"] + "\"" + " by " + next[1]["artist"] + "\n" + "on: " + next[1]["album"])
@@ -520,16 +559,18 @@ class Panda(QtGui.QMainWindow):
 			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/Thumbs-up-right-trans.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(0, 0, 0);")
 
 		self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/playing-trans.png);\nborder: rgb(0, 0, 0);\nbackground-color: "+ self.c.color + ";")
+		self.ui.btnNext.setEnabled( True )
 
-	def skipSong( self ):
+	def skipSong( self ): #Skip song
+		self.ui.btnNext.setEnabled( False )
 		self.player.skip = True
 		self.skip = 0
 		self.player.stopPlayBack()
 
-	def stop( self ):
+	def stop( self ): #Stop Playing, not currently implemented
 		self.player.playing = False
 
-	def closeEvent(self, event=None):
+	def closeEvent(self, event=None): #Exit application gracefully.. somewhat.
 		self.player.skip = False
 		if self.player.playing:
 			self.player.playing = False
