@@ -1,3 +1,4 @@
+#Import, of course
 import sys, time
 
 from libpiano.pandora import Pandora
@@ -6,15 +7,14 @@ from PyQt4 import QtCore
 class PandaException( Exception ):
 	pass
 
-class RadioPlayer(QtCore.QThread):
+class RadioPlayer(QtCore.QThread): #Main radio class
 	__pyqtSignals__ = ( "PlayNext", "LoveHate", "songs", "stations", "deleted", "tired", "authSuccess", \
-		"stationCreated", "authFail", "duration", "state(QString)" )
+		"stationCreated", "authFail", "duration", "state(QString)", "seed" )
 	
 	def __init__( self, Parent=None ):
 		self.timer = None
 		self.playing = 0
 		self.skip = False
-		self.eof = False
 		self.run = 1
 		self.user = ""
 		self.password = ""
@@ -22,6 +22,7 @@ class RadioPlayer(QtCore.QThread):
 		self.duration = 0
 		self.position = 0
 
+		self.seedSong = 0
 		self.getStatlist = 0
 		self.getPlaylist = 0
 		self.loveHateSong = 0
@@ -34,7 +35,7 @@ class RadioPlayer(QtCore.QThread):
 		self.createStationId = 0
 		
 		self.os = self.get_product()
-		if self.os == "RX-51":
+		if self.os == "RX-51": #Check for N900 or Diablo
 			from libpiano.MAFWBackend import MAFWBackend
 			self.player = MAFWBackend()
 		else:
@@ -55,7 +56,7 @@ class RadioPlayer(QtCore.QThread):
 		QtCore.QThread.__init__(self)
 
 	def run(self):
-		while self.run:
+		while self.run: #Run a constant loop and see if the main UI class has asked us to do anything
 			if self.loveHateSong:
 				self.loveHateSong = 0
 				self.LoveHate()
@@ -81,10 +82,13 @@ class RadioPlayer(QtCore.QThread):
 			if self.createStationId>0:
 				self.createStation()
 				self.createStationId = 0
+			if self.seedSong!=0:
+				self.doSeed(self.seedSong)
+				self.seedSong = 0
 
 			time.sleep ( 0.5 )
 
-	def get_product( self ):
+	def get_product( self ): #Find Product ID
 		try:                                                               
 			lines = open("/proc/component_version", "r").readlines()       
 		except:                                                            
@@ -100,7 +104,7 @@ class RadioPlayer(QtCore.QThread):
 		#end for
 		return product
 
-	def updateTime( self, pos, total ):
+	def updateTime( self, pos, total ): #Update position/duration
 		if self.playing:
 			self.position = pos
 			self.duration = total
@@ -108,22 +112,22 @@ class RadioPlayer(QtCore.QThread):
 			if self.duration > 0:
 				self.emit(QtCore.SIGNAL("duration"))
 
-	def stateChange( self, state ):
+	def stateChange( self, state ): #Update backend state change
 		if state == 2:
 			self.emit(QtCore.SIGNAL("state(QString)"),"pause")
 		elif state == 1:
 			self.emit(QtCore.SIGNAL("state(QString)"),"play")
 
-	def createStation( self ):
+	def createStation( self ): #Create a station at Pandora
 		ret = self.pandora.createStation( self.createStationId )
 		self.playStation = ret["stationId"]
 		self.emit(QtCore.SIGNAL("stationCreated"))
 
-	def search( self, search ):
+	def search( self, search ): #Search Pandora
 		ret = self.pandora.searchPandora( str(search) )
 		return ret
 
-	def auth( self, user, password ):
+	def auth( self, user, password ): #Authenticate to Pandora
 		ret = self.pandora.authListener( str(user), str(password) )
 		print ret
 		if ret:
@@ -133,61 +137,61 @@ class RadioPlayer(QtCore.QThread):
 			self.success = 0
 			self.emit(QtCore.SIGNAL("authFail"))
 
-	def playSong( self, item ):
+	def playSong( self, item ): #Start playing
 		self.play( item[0], item[1] )
 
-	def play( self, url, item ):
-		time.sleep ( 2 )
+	def play( self, url, item ): #Load URL/Song into Backend
 		self.player.load_audio(url)
-		if self.eof:
-			self.eof = False
 
-	def stopPlayBack ( self,state=None ):
+	def stopPlayBack ( self,state=None ): #Pause or Stop
 		if state == 'pause':
 			self.player.pause()
 		else:
 			self.player.stop()
-			if self.skip:
-				self.onPlayBackStopped()
+			if self.os == "RX-51":
+				self.player.close()
+			if self.skip and self.playing:
+				self.onPlayBackEnded()
 	
-	def startPlayBack ( self ):
+	def startPlayBack ( self ): #Resume after Pause
 		self.player.play()
 
-	def onPlayBackStopped( self ):
-		if self.playing:
-			if self.skip:
-				self.skip = False
-				self.emit(QtCore.SIGNAL("PlayNext"))
+	def onPlayBackEnded( self ): #EOF Reached, start next song.
+		print "EOF called."
+		self.emit(QtCore.SIGNAL("PlayNext"))
+		time.sleep( 2 )
 
-	def onPlayBackEnded( self ):
-		if not self.eof:
-			if self.skip:
-				self.skip = False
-			self.eof = True
-			self.emit(QtCore.SIGNAL("PlayNext"))
-
-	def LoveHate( self ):
+	def LoveHate( self ): #Do Love/Hate based on variable settings
 		self.respLH = self.pandora.addFeedback( self.curSong[1]["stationId"], self.curSong, self.loveHateRating )
 		self.emit(QtCore.SIGNAL("LoveHate"))
 
-	def doTired( self ):
-		self.respTired = self.pandora.setTired( self.curSong[1]["identity"] )
-		print self.respTired
+	def doSeed( self, artist=None ):
+		if artist == "artist":
+			musicId = self.curSong[1]["artistMusicId"]
+		else:
+			musicId = self.curSong[1]["musicId"]
+		respLH = self.pandora.addSeed( self.curSong[1]["stationId"], musicId )
+		self.emit(QtCore.SIGNAL("seed"))
+		#print respLH
+
+	def doTired( self ): #Mark tired
+		respTired = self.pandora.setTired( self.curSong[1]["identity"] )
+		print respTired
 		self.emit(QtCore.SIGNAL("tired"))
 
-	def doDelete( self ):
-		self.respDelete = self.pandora.deleteStation( self.curStation )
-		print self.respDelete
-		if self.respDelete:
+	def doDelete( self ): #Delete station
+		respDelete = self.pandora.deleteStation( self.curStation )
+		print respDelete
+		if respDelete:
 			self.emit(QtCore.SIGNAL("deleted"))
 			self.getStations()
 
-	def getStations( self ):
-		self.stations = self.pandora.getStations()
+	def getStations( self ): #Get station list, sort alphabetically.
+		self.stations = sorted(self.pandora.getStations(), key=lambda station: station["stationName"])
 		self.emit(QtCore.SIGNAL("stations"))
 
-	def getMoreSongs( self ):
-		if self.curStation == "":
+	def getMoreSongs( self ): #Get next 3 or so songs
+		if self.curStation == "": #Can't get songs without a station.  This shouldn't happen.
 			raise PandaException()
 		items = []
 		fragment = self.pandora.getFragment( self.curStation, self.format )
@@ -206,11 +210,12 @@ class RadioPlayer(QtCore.QThread):
 			"focusTraitId"	: s["focusTraitId"], \
 			"stationId"	: s["stationId"], \
 			"identity"	: s["identity"], \
-			"musicId"	: s["musicId"] }
+			"musicId"	: s["musicId"], \
+			"artistMusicId" : s["artistMusicId"] }
 			items.append( ( s["audioURL"], item ) )
 		self.playlist.extend( items )
 
-	def close( self ):
+	def close( self ): #Close Pandora and Backend
 		self.player.stop()
 		self.player.close()
 		self.player = None
